//! # Structures de parsing des données de la fonction SSV_LireCartePS

use deku::deku_derive;

use crate::fsv_parsing::groups::NumericString;

/// # Titulaire
/// 1 occurence
pub mod group_1_holder {

    use crate::fsv_parsing::groups::{ extract_enum_id_from_str, AlphaNumericString };

    use super::*;

    /// Groupe 1 - Titulaire
    #[deku_derive(DekuRead)]
    #[derive(Debug, PartialEq)]
    pub struct Holder {
        #[deku(temp)]
        card_type_raw: NumericString,
        #[deku(ctx = "extract_enum_id_from_str::<u8>(&*card_type_raw.0, 255)")]
        pub card_type: CardPSType,
        #[deku(temp)]
        national_id_type_raw: NumericString,
        #[deku(ctx = "extract_enum_id_from_str::<u8>(&*national_id_type_raw.0, 255)")]
        pub national_id_type: NationalIDType,
        // TODO: handle national_id depending on national_id_type
        pub national_id: AlphaNumericString, // /!\ CE and not CA
        pub national_id_key: AlphaNumericString,
        #[deku(temp)]
        civility_code_raw: NumericString,
        #[deku(ctx = "extract_enum_id_from_str::<u8>(&*civility_code_raw.0, 255)")]
        pub civility_code: CivilityCode,
        pub holder_lastname: AlphaNumericString, // /!\ CE and not CA
        pub holder_firstname: AlphaNumericString, // /!\ CE and not CA
        #[deku(temp)]
        category_card_size: u8,
        pub category_card: CategoryCard,
    }

    // Fields
    #[deku_derive(DekuRead)]
    #[derive(Debug, PartialEq)]
    #[deku(ctx = "id: u8", id = "id")]
    pub enum CardPSType {
        #[deku(id = 0)]
        CPS, // Carte de Professionnel de Santé (CPS)
        #[deku(id = 1)]
        CPF, // Carte de Professionnel de Santé en Formation (CPF)
        #[deku(id = 2)]
        CPE, // Carte de Personnel / Directeur⋅ice d'Établissement de Santé (CDE/CPE)
        #[deku(id = 3)]
        CPA, // Carte de Personnel / Directeur⋅ice Autorisé⋅e (CDA/CPA)
        #[deku(id = 4)]
        CPM, // Carte de Personne Morale
    }

    #[deku_derive(DekuRead)]
    #[derive(Debug, PartialEq)]
    #[deku(ctx = "id: u8", id = "id")]
    pub enum NationalIDType {
        #[deku(id = 0)]
        ADELI, // N° ADELI
        #[deku(id = 1)]
        ADELICabinet, // Id Cabinet ADELI + N° employé
        #[deku(id = 2)]
        DRASS, // N° DRASS
        #[deku(id = 3)]
        FINESS, // N° FINESS + N° employé
        #[deku(id = 4)]
        SIREN, // N° SIREN + N° employé
        #[deku(id = 5)]
        SIRET, // N° SIRET + N° employé
        #[deku(id = 6)]
        RPPSCabinet, // Id Cabinet RPPS + N° employé
        #[deku(id = 8)]
        RPPS, // N° RPPS
        #[deku(id = 9)]
        ADELIEtudiantMedecin, // N° Etudiant Médecin type ADELI sur 9 caractères (information transmise par l’ANS)
    }

    #[deku_derive(DekuRead)]
    #[derive(Debug, PartialEq)]
    #[deku(ctx = "id: u8", id = "id")]
    pub enum CivilityCode {
        #[deku(id = 1)]
        Adjudant,
        #[deku(id = 2)]
        Amiral,
        #[deku(id = 3)]
        Aspirant,
        #[deku(id = 4)]
        Aumonier,
        #[deku(id = 5)]
        Capitaine,
        #[deku(id = 6)]
        Cardinal,
        #[deku(id = 7)]
        Chanoine,
        #[deku(id = 8)]
        Colonel,
        #[deku(id = 9)]
        Commandant,
        #[deku(id = 10)]
        Commissaire,
        #[deku(id = 11)]
        Conseiller,
        #[deku(id = 12)]
        Directeur,
        #[deku(id = 13)]
        Docteur,
        #[deku(id = 14)]
        Douanier,
        #[deku(id = 15)]
        Epouxse,
        #[deku(id = 16)]
        Eveque,
        #[deku(id = 17)]
        General,
        #[deku(id = 18)]
        Gouverneur,
        #[deku(id = 19)]
        Ingenieur,
        #[deku(id = 20)]
        Inspecteur,
        #[deku(id = 21)]
        Lieutenant,
        #[deku(id = 22)]
        Madame,
        #[deku(id = 23)]
        Mademoiselle,
        #[deku(id = 24)]
        Maitre,
        #[deku(id = 25)]
        Marechal,
        #[deku(id = 26)]
        Medecin,
        #[deku(id = 27)]
        Mesdames,
        #[deku(id = 28)]
        Mesdemoiselles,
        #[deku(id = 29)]
        Messieurs,
        #[deku(id = 30)]
        Monseigneur,
        #[deku(id = 31)]
        Monsieur,
        #[deku(id = 32)]
        NotreDame,
        #[deku(id = 33)]
        Pasteur,
        #[deku(id = 34)]
        Prefet,
        #[deku(id = 35)]
        President,
        #[deku(id = 36)]
        Professeur,
        #[deku(id = 37)]
        Recteur,
        #[deku(id = 38)]
        Sergent,
        #[deku(id = 39)]
        SousPrefet,
        #[deku(id = 40)]
        Technicien,
        #[deku(id = 41)]
        Veuve,
    }

    #[deku_derive(DekuRead)]
    #[derive(Debug, PartialEq)]
    #[deku(id_type="u8")]
    pub enum CategoryCard {
        #[deku(id = 74)] // T
        Test,
        #[deku(id = 72)] // R
        Reelle,
        #[deku(id_pat = "_")]
        Unknown,
    }
}

#[cfg(test)]
mod tests {
    use deku::DekuContainerRead as _;
    use group_1_holder::{CardPSType, CategoryCard, CivilityCode, NationalIDType};

    use crate::fsv_parsing::blocks::BlockHeader;

    use super::*;

    mod data {
        pub const BUFFER: &[u8] = &[
            0, 1, // Block ID
                53, // Block size
                    1, // Type de carte PS, 2 CN
                        48, // 0
                    1, // Type d'identification nationale, 1 CN
                        56, // 8
                    11, // N° d'identification nationale, 8-30 CE
                        57, 57, 55, 48, 48, 53, 57, 51, 54, 56, 54,
                    1, // Clé du N° d'identification nationale, 1 CN
                        54, // 6
                    2, // Code civilité, 2 CN
                        51, 49, // 31
                    23, // Nom du PS, 27 CE
                        80, 72, 65, 82, 77, 79, 70, 70, 73, 67,
                        69, 32, 82, 80, 80, 83, 48, 48, 53, 57,
                        51, 54, 56,
                    7, // Prénom du PS, 27 CE
                        71, 73, 76, 66, 69, 82, 84,
                    // ??? Missing ??? Catégorie Carte, 1 CA
            0, 2, // Block ID
                93, // Block size
                    1, // N° logique de la situation de facturation du PS, 1 CB
                        1,
                    1, // Mode d'exercice, 2 CN
                        48,
                    1, // Statut d'exercice, 3 CN
                        49,
                    2, // Secteur d'activité, 3 CN
                        56, 54,
                    1, // Type d'identification structure, 1 CN
                        49,
                    9, // N° d'identification structure, 14 CA
                        48, 66, 48, 50, 52, 54, 50, 56, 54,
                    1, // Clé du N° d'identification structure, 1 CN
                        54,
                    34, // Raison sociale structure, 40 CE
                        80, 72, 65, 82, 77, 65, 67, 73, 69, 32,
                        68, 69, 32, 76, 65, 32, 71, 65, 82, 69,
                        32, 82, 79, 85, 84, 73, 69, 82, 69, 50,
                        52, 54, 50, 56,
                    8, // N° d'identification de facturation du PS, 8 CN
                        48, 48, 50, 48, 57, 51, 54, 56,
                    1, // Clé du N° d'identification de facturation du PS, 1 CN
                        48,
                    0, // N° d'identification du PS remplaçant, 30 CA
                    1, // Clé du N° d'identification du PS remplaçant, 1 CN
                        48,
                    1, // Code conventionnel, 1 CN
                        49,
                    2, // Code spécialité, 2 CN
                        53, 48,
                    2, // Code zone tarifaire, 2 CN
                        49, 48,
                    2, // Code zone IK
                        48, 48,
                    1, // Code agrément 1, 1 CN
                        48,
                    1, // Code agrément 2, 1 CN
                        48,
                    1, // Code agrément 3, 1 CN
                        48,
                    1, // Habilitation à signer une Facture // 1 CN
                        49,
                    1, // Habilitation à signer un lot // 1 CN
                        49
        ];
    }

    #[test]
    fn test_lire_carte_ps_first_header() {
        // env_logger::init(); // Uncomment and run with RUST_LOG=trace for deku debugging
        let ((_rest, _offset), block_header) = BlockHeader::from_bytes((data::BUFFER, 0)).unwrap();
        assert_eq!(block_header.group_id.0, 1, "Header ID");
        assert_eq!(block_header.data_size, 53, "Header Size");
    }

    #[test]
    fn test_group_1_holder() {
        env_logger::init(); // Uncomment and run with RUST_LOG=trace for deku debugging
        let offset = 3*8;
        let (_rest, holder) = group_1_holder::Holder::from_bytes((data::BUFFER, offset)).unwrap();
        assert_eq!(holder.card_type, CardPSType::CPS, "Card type");
        assert_eq!(holder.national_id_type, NationalIDType::RPPS, "National ID type");
        assert_eq!(holder.national_id.0, "99700593686", "National Id");
        assert_eq!(holder.national_id_key.0, "6", "National ID Key");
        assert_eq!(holder.civility_code, CivilityCode::Monsieur, "Civility Code");
        assert_eq!(holder.holder_lastname.0, "PHARMOFFICE RPPS0059368", "Holder Lastname");
        assert_eq!(holder.holder_firstname.0, "GILBERT", "Holder Firstname");
        assert_eq!(holder.category_card, CategoryCard::Unknown, "Category card");
    }
}